"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
from typing import Union

from . import utils
from .client_hints import ClientHints
from .data import DEVICES, BROWSERS, PLATFORMS, PLATFORMS_DESKTOP, PLATFORMS_MOBILE, T_DEVICES, T_PLATFORMS, T_BROWSERS
from .data.generator import Generator
from .headers import Headers
from .options import Options


class UserAgent:
    def __init__(self,
                 device: Union[T_DEVICES, tuple, list, None] = None,
                 platform: Union[T_PLATFORMS, tuple, list, None] = None,
                 browser: Union[T_BROWSERS, tuple, list, None] = None,
                 options: Union[Options, None] = None):

        self.device: Union[str, None] = utils.choice(device) if device else None
        self.platform: Union[str, None] = utils.choice(platform) if platform else None
        self.browser: Union[str, None] = utils.choice(browser) if browser else None
        self.options: Options = options if options else Options()
        self.__complete()

        # Type hinting only
        self.text: str
        self.ch: ClientHints
        self.headers: Headers
        self.generator: Generator

    def __find_device(self) -> str:
        if self.device is not None and self.device not in DEVICES:
            raise ValueError('No such device type found: {}'.format(self.device))

        # Override the device type, if the platform is specified
        if self.platform is not None:
            if self.platform in PLATFORMS_DESKTOP:
                self.device = 'desktop'
            elif self.platform in PLATFORMS_MOBILE:
                self.device = 'mobile'

        if self.device is None:
            self.device = utils.choice(DEVICES)

        return self.device

    def __find_platform(self) -> str:
        if self.platform is not None and self.platform not in PLATFORMS:
            raise ValueError('No such platform found: {}'.format(self.platform))

        # Make the platform consistent with the device type and browser
        if self.device == 'desktop' and self.platform not in PLATFORMS_DESKTOP:
            # Safari only supports the macOS and iOS platforms
            if self.browser is not None and self.browser == 'safari':
                self.platform = utils.choice(('macos', 'ios'))
            else:
                self.platform = utils.choice(PLATFORMS_DESKTOP)
        elif self.device == 'mobile' and self.platform not in PLATFORMS_MOBILE:
            self.platform = utils.choice(PLATFORMS_MOBILE)

        if self.platform is None:
            self.platform = utils.choice(PLATFORMS)

        return self.platform

    def __find_browser(self) -> str:
        if self.browser is not None and self.browser not in BROWSERS:
            raise ValueError('No such browser found: {}'.format(self.browser))

        if self.browser is None:
            self.browser = utils.choice(BROWSERS)

        # Safari only supports the macOS and iOS platforms
        if self.browser == 'safari' and self.platform not in ('macos', 'ios'):
            self.browser = 'chrome'

        return self.browser

    def __complete(self):
        self.device = self.__find_device()
        self.platform = self.__find_platform()
        self.browser = self.__find_browser()

        ua = Generator(device=self.device, platform=self.platform, browser=self.browser, options=self.options)
        self.text = ua.user_agent
        self.ch = ClientHints(ua)
        self.headers = Headers(ua, self.ch)
        self.generator = ua

    def __str__(self):
        return self.text

    def __repr__(self):
        return f"UserAgent(device='{self.device}', platform='{self.platform}', browser='{self.browser}')"



"""
Random User-Agent
Copyright: 2025 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

from src.ua_generator.data.browsers.chrome import versions as chrome_versions
from src.ua_generator.data.filterer import Filterer
from src.ua_generator.data.version import VersionRange


class TestFilterer(unittest.TestCase):
    def test_unfiltered(self):
        filterer = Filterer(chrome_versions)
        self.assertEqual(len(filterer.versions), len(chrome_versions))

    def test_version_range(self):
        CHROME_MIN = 131
        CHROME_MAX = 135

        filterer = Filterer(chrome_versions)
        filterer.version_range(VersionRange(CHROME_MIN, CHROME_MAX))
        self.assertEqual(len(filterer.versions), (CHROME_MAX - CHROME_MIN) + 1)

    def test_weighted_versions(self):
        filterer = Filterer(chrome_versions)
        filterer.weighted_versions()
        self.assertEqual(len(filterer.versions), 1)


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
from typing import Literal

DEVICES = ('desktop', 'mobile')
T_DEVICES = Literal['desktop', 'mobile']

PLATFORMS = ('windows', 'macos', 'ios', 'linux', 'android')
T_PLATFORMS = Literal['windows', 'macos', 'ios', 'linux', 'android']
PLATFORMS_DESKTOP = ('windows', 'macos', 'linux')  # Platforms on desktop devices
PLATFORMS_MOBILE = ('ios', 'android')  # Platforms on mobile devices

BROWSERS = ('chrome', 'edge', 'firefox', 'safari')
T_BROWSERS = Literal['chrome', 'edge', 'firefox', 'safari']
BROWSERS_SUPPORT_CH = ('chrome', 'edge')  # Browsers that support Client Hints



"""
Random User-Agent
Copyright: 2022-2025 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""
import random
from typing import List

from ...filterer import Filterer
from ...version import Version, AndroidVersion
from ....options import Options

# https://en.wikipedia.org/wiki/Android_version_history
# https://source.android.com/setup/start/build-numbers
versions: List[AndroidVersion] = [
    AndroidVersion(Version(major=8, minor=0, build=0), build_numbers=('OPR6.{d}.{v}', 'OPR5.{d}.{v}', 'OPR4.{d}.{v}', 'OPR3.{d}.{v}', 'OPR2.{d}.{v}', 'OPR1.{d}.{v}', 'OPD3.{d}.{v}', 'OPD2.{d}.{v}', 'OPD1.{d}.{v}')),
    AndroidVersion(Version(major=8, minor=1, build=0), build_numbers=('OPM8.{d}.{v}', 'OPM7.{d}.{v}', 'OPM6.{d}.{v}', 'OPM5.{d}.{v}', 'OPM4.{d}.{v}', 'OPM3.{d}.{v}', 'OPM2.{d}.{v}')),
    AndroidVersion(Version(major=9, minor=0, build=0), build_numbers=('PQ3B.{d}.{v}', 'PQ3A.{d}.{v}', 'PQ2A.{d}.{v}', 'PQ1A.{d}.{v}', 'PPR2.{d}.{v}', 'PPR1.{d}.{v}')),
    AndroidVersion(Version(major=10, minor=0, build=0), build_numbers=('QD4A.{d}.{v}', 'QQ3A.{d}.{v}', 'QQ2A.{d}.{v}', 'QQ1D.{d}.{v}', 'QQ1C.{d}.{v}', 'QQ1B.{d}.{v}', 'QQ1A.{d}.{v}', 'QP1A.{d}.{v}', 'QD1A.{d}.{v}')),
    AndroidVersion(Version(major=11, minor=0, build=0), build_numbers=('RD2A.{d}.{v}', 'RQ3A.{d}.{v}', 'RQ2A.{d}.{v}', 'RQ1D.{d}.{v}', 'RQ1C.{d}.{v}', 'RQ1A.{d}.{v}', 'RD1B.{d}.{v}', 'RD1A.{d}.{v}', 'RP1A.{d}.{v}')),
    AndroidVersion(Version(major=12, minor=0, build=0), build_numbers=('SP1A.{d}.{v}', 'SQ1D.{d}.{v}', 'SD1A.{d}.{v}')),
    AndroidVersion(Version(major=12, minor=1, build=0), build_numbers=('SP2A.{d}.{v}', 'SD2A.{d}.{v}', 'SQ3A.{d}.{v}')),
    AndroidVersion(Version(major=13, minor=0, build=0), build_numbers=('TQ3A.{d}.{v}', 'TQ3C.{d}.{v}', 'TQ2B.{d}.{v}', 'TD4A.{d}.{v}', 'TQ2A.{d}.{v}', 'TQ1A.{d}.{v}', 'TD1A.{d}.{v}', 'TP1A.{d}.{v}')),
    AndroidVersion(Version(major=14, minor=0, build=0), build_numbers=('AP2A.{d}.{v}', 'AD1A.{d}.{v}', 'UD2A.{d}.{v}', 'UQ1A.{d}.{v}', 'UP1A.{d}.{v}', 'UD1A.{d}.{v}')),
    AndroidVersion(Version(major=15, minor=0, build=0), build_numbers=('AP4A.{d}.{v}', 'AP3A.{d}.{v}')),
]

platform_models = ('Pixel 2', 'Pixel 2 XL', 'Pixel 3', 'Pixel 3a', 'Pixel 3a XL', 'Pixel 3 XL', 'Pixel 4',
                   'Pixel 4 XL', 'Pixel 4a (5G)', 'Pixel 5', 'Pixel 5a (5G)', 'Pixel 6', 'Pixel 6 Pro',
                   'Pixel 6a', 'Pixel 7', 'Pixel 7 Pro', 'Pixel 8', 'Pixel 8 Pro', 'Pixel 8a')


def get_version(options: Options) -> AndroidVersion:
    filterer = Filterer(versions)

    if options.version_ranges and 'android' in options.version_ranges:
        filterer.version_range(options.version_ranges['android'])

    if options.weighted_versions:
        filterer.weighted_versions(max_range=4)

    choice: AndroidVersion = random.choice(filterer.versions)
    choice.build_number = choice.build_number.replace('{d}', '{:02d}{:02d}{:02d}'.format(random.randint(17, 25), random.randint(0, 12), random.randint(0, 29)))
    choice.build_number = choice.build_number.replace('{v}', '{}'.format(random.randint(1, 255)))
    choice.platform_model = random.choice(platform_models)
    return choice



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import random
from typing import List

from ..filterer import Filterer
from ..version import Version, ChromiumVersion
from ...options import Options

# https://docs.microsoft.com/en-us/deployedge/microsoft-edge-release-schedule
versions: List[ChromiumVersion] = [
    ChromiumVersion(Version(major=100, minor=0, build=1185, patch=(0, 99))),
    ChromiumVersion(Version(major=101, minor=0, build=1210, patch=(0, 99))),
    ChromiumVersion(Version(major=102, minor=0, build=1245, patch=(0, 99))),
    ChromiumVersion(Version(major=103, minor=0, build=1264, patch=(0, 99))),
    ChromiumVersion(Version(major=104, minor=0, build=1293, patch=(0, 99))),
    ChromiumVersion(Version(major=105, minor=0, build=1343, patch=(0, 99))),
    ChromiumVersion(Version(major=106, minor=0, build=1370, patch=(0, 99))),
    ChromiumVersion(Version(major=107, minor=0, build=1418, patch=(0, 99))),
    ChromiumVersion(Version(major=108, minor=0, build=1462, patch=(0, 99))),
    ChromiumVersion(Version(major=109, minor=0, build=1518, patch=(0, 99))),
    ChromiumVersion(Version(major=110, minor=0, build=1587, patch=(0, 99))),
    ChromiumVersion(Version(major=111, minor=0, build=1661, patch=(0, 99))),
    ChromiumVersion(Version(major=112, minor=0, build=1722, patch=(0, 99))),
    ChromiumVersion(Version(major=113, minor=0, build=1774, patch=(0, 99))),
    ChromiumVersion(Version(major=114, minor=0, build=1823, patch=(0, 99))),
    ChromiumVersion(Version(major=115, minor=0, build=1901, patch=(0, 99))),
    ChromiumVersion(Version(major=116, minor=0, build=1938, patch=(0, 99))),
    ChromiumVersion(Version(major=117, minor=0, build=2045, patch=(0, 99))),
    ChromiumVersion(Version(major=118, minor=0, build=2088, patch=(0, 99))),
    ChromiumVersion(Version(major=119, minor=0, build=2151, patch=(0, 99))),
    ChromiumVersion(Version(major=120, minor=0, build=2210, patch=(0, 99))),
    ChromiumVersion(Version(major=121, minor=0, build=2277, patch=(0, 99))),
    ChromiumVersion(Version(major=122, minor=0, build=2365, patch=(0, 99))),
    ChromiumVersion(Version(major=123, minor=0, build=2420, patch=(0, 99))),
    ChromiumVersion(Version(major=124, minor=0, build=2478, patch=(0, 99))),
    ChromiumVersion(Version(major=125, minor=0, build=2535, patch=(0, 99))),
    ChromiumVersion(Version(major=126, minor=0, build=2592, patch=(0, 99))),
    ChromiumVersion(Version(major=127, minor=0, build=2651, patch=(0, 99))),
    ChromiumVersion(Version(major=128, minor=0, build=2739, patch=(0, 99))),
    ChromiumVersion(Version(major=129, minor=0, build=2792, patch=(0, 99))),
    ChromiumVersion(Version(major=130, minor=0, build=2849, patch=(0, 99))),
    ChromiumVersion(Version(major=131, minor=0, build=2903, patch=(0, 99))),
    ChromiumVersion(Version(major=132, minor=0, build=2957, patch=(0, 99))),
    ChromiumVersion(Version(major=133, minor=0, build=3065, patch=(0, 99))),
    ChromiumVersion(Version(major=134, minor=0, build=3124, patch=(0, 99))),
    ChromiumVersion(Version(major=135, minor=0, build=3179, patch=(0, 99))),
    ChromiumVersion(Version(major=136, minor=0, build=3240, patch=(0, 99))),
]


def get_version(options: Options) -> ChromiumVersion:
    filterer = Filterer(versions)

    if options.version_ranges and 'edge' in options.version_ranges:
        filterer.version_range(options.version_ranges['edge'])

    if options.weighted_versions:
        filterer.weighted_versions()

    return random.choice(filterer.versions)



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""
import random
from typing import Union


def choice(t: Union[str, tuple, list, None]) -> Union[str, None]:
    if type(t) is str:
        return t
    if type(t) is tuple or type(t) is list:
        return random.choice(t)

    return None



"""
Random User-Agent
Copyright: 2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

import src.ua_generator as ua_generator
from src.ua_generator.data.version import VersionRange
from src.ua_generator.options import Options


class TestOptions(unittest.TestCase):
    def test_weighted_versions(self):
        ua = ua_generator.generate()
        self.assertFalse(ua.options.weighted_versions)
        ua = ua_generator.generate(options=Options(weighted_versions=True))
        self.assertTrue(ua.options.weighted_versions)
        ua = ua_generator.generate(options=Options(weighted_versions=False))
        self.assertFalse(ua.options.weighted_versions)

    def test_version_ranges(self):
        ua = ua_generator.generate()
        self.assertIsNone(ua.options.version_ranges)

        options = Options()
        options.version_ranges={'chrome': VersionRange(125, 127)}

        for i in range(0, 100):
            ua = ua_generator.generate(browser='chrome', options=options)
            self.assertIn(ua.generator.browser_version.major, (125, 126, 127))


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022-2025 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""
import random
from typing import List

from ...filterer import Filterer
from ...version import Version, AndroidVersion
from ....options import Options

# https://en.wikipedia.org/wiki/Android_version_history
# https://source.android.com/setup/start/build-numbers
versions: List[AndroidVersion] = [
    AndroidVersion(Version(major=8, minor=0, build=0), build_numbers=('OPR6.{d}.{v}', 'OPR5.{d}.{v}', 'OPR4.{d}.{v}', 'OPR3.{d}.{v}', 'OPR2.{d}.{v}', 'OPR1.{d}.{v}', 'OPD3.{d}.{v}', 'OPD2.{d}.{v}', 'OPD1.{d}.{v}')),
    AndroidVersion(Version(major=8, minor=1, build=0), build_numbers=('OPM8.{d}.{v}', 'OPM7.{d}.{v}', 'OPM6.{d}.{v}', 'OPM5.{d}.{v}', 'OPM4.{d}.{v}', 'OPM3.{d}.{v}', 'OPM2.{d}.{v}')),
    AndroidVersion(Version(major=9, minor=0, build=0), build_numbers=('PQ3B.{d}.{v}', 'PQ3A.{d}.{v}', 'PQ2A.{d}.{v}', 'PQ1A.{d}.{v}', 'PPR2.{d}.{v}', 'PPR1.{d}.{v}')),
    AndroidVersion(Version(major=10, minor=0, build=0), build_numbers=('QD4A.{d}.{v}', 'QQ3A.{d}.{v}', 'QQ2A.{d}.{v}', 'QQ1D.{d}.{v}', 'QQ1C.{d}.{v}', 'QQ1B.{d}.{v}', 'QQ1A.{d}.{v}', 'QP1A.{d}.{v}', 'QD1A.{d}.{v}')),
    AndroidVersion(Version(major=11, minor=0, build=0), build_numbers=('RD2A.{d}.{v}', 'RQ3A.{d}.{v}', 'RQ2A.{d}.{v}', 'RQ1D.{d}.{v}', 'RQ1C.{d}.{v}', 'RQ1A.{d}.{v}', 'RD1B.{d}.{v}', 'RD1A.{d}.{v}', 'RP1A.{d}.{v}')),
    AndroidVersion(Version(major=12, minor=0, build=0), build_numbers=('SP1A.{d}.{v}', 'SQ1D.{d}.{v}', 'SD1A.{d}.{v}')),
    AndroidVersion(Version(major=12, minor=1, build=0), build_numbers=('SP2A.{d}.{v}', 'SD2A.{d}.{v}', 'SQ3A.{d}.{v}')),
    AndroidVersion(Version(major=13, minor=0, build=0), build_numbers=('TQ3A.{d}.{v}', 'TQ3C.{d}.{v}', 'TQ2B.{d}.{v}', 'TD4A.{d}.{v}', 'TQ2A.{d}.{v}', 'TQ1A.{d}.{v}', 'TD1A.{d}.{v}', 'TP1A.{d}.{v}')),
    AndroidVersion(Version(major=14, minor=0, build=0), build_numbers=('AP2A.{d}.{v}', 'AD1A.{d}.{v}', 'UD2A.{d}.{v}', 'UQ1A.{d}.{v}', 'UP1A.{d}.{v}', 'UD1A.{d}.{v}')),
    AndroidVersion(Version(major=15, minor=0, build=0), build_numbers=('AP4A.{d}.{v}', 'AP3A.{d}.{v}')),
]


# https://whatmyuseragent.com/brand/xi/xiaomi
# https://gist.github.com/iamdual/2ef10eeae2c3ce22470bb9acfa77435e
platform_models = (
    '11i HyperCharge 5G', '12', '12 Lite', '12 Pro', '12 Pro Dimensity', '12S', '12S Pro', '12S Ultra', '12T', '12T Pro',
    '12X', '13 Lite', '13 Pro', '13 Ultra', '13T', '13T Pro', '14', '14 Civi', '14 Pro', '14 Pro Ti', '14 Ultra', '14T Pro',
    '15', '15 Pro', 'Black Shark', 'Black Shark 2', 'Black Shark 2 Pro', 'Black Shark 3', 'Black Shark 3 5G',
    'Black Shark 3 Pro', 'Black Shark 3 Pro 5G', 'Black Shark 4', 'Black Shark 4 Pro', 'Black Shark 5', 'Black Shark 5 Pro',
    'Black Shark Helo', 'CC11', 'Civi', 'Civi 1S', 'Civi 2', 'Civi 3', 'Civi 4', 'Mi 10', 'Mi 10 Lite 5G', 'Mi 10 Pro',
    'Mi 10 Ultra 5G', 'Mi 10i', 'Mi 10S', 'Mi 10T 5G', 'Mi 10T Lite 5G', 'Mi 10T Pro 5G', 'Mi 11', 'Mi 11 Lite',
    'Mi 11 Lite 5G', 'Mi 11 Lite 5G NE', 'Mi 11 Pro', 'Mi 11 Ultra', 'Mi 11i', 'Mi 11T', 'Mi 11T Pro', 'Mi 11X',
    'Mi 11X Pro', 'Mi 4 LTE', 'Mi 4i', 'Mi 4W', 'Mi 5s Plus', 'Mi 5X', 'Mi 8', 'Mi 8 Explorer Edition', 'Mi 8 Lite', 'Mi 9',
    'Mi 9 Lite', 'Mi 9 Pro 5G', 'Mi 9 SE', 'Mi 9 Transparent Edition', 'Mi 9T Pro', 'Mi A2 Lite', 'Mi A3', 'Mi CC 9',
    'Mi CC 9 Pro', 'Mi CC 9 Pro Premium Edition', 'Mi CC 9e', 'Mi Laser Projector 150"', 'Mi Max', 'Mi Max 3 Pro', 'Mi Mix',
    'Mi Mix 2', 'Mi Mix 2S', 'Mi Mix 2S Art', 'Mi Mix 3 5G', 'Mi Mix 4', 'Mi Mix Fold', 'Mi Note 10', 'Mi Note 10 Lite',
    'Mi Note 10 Pro', 'Mi Note 3', 'Mi Note Pro', 'Mi One Plus', 'Mi Pad', 'Mi Pad 2', 'Mi Pad 3', 'Mi Pad 4',
    'Mi Pad 4 Plus', 'Mi Pad 4 WiFi', 'Mi Pad 5', 'Mi Pad 5 Pro', 'Mi Pad 5 Pro 5G', 'Mi Play',
    'Mi Smart Compact Projector', 'Mix Flip', 'Mix Fold 2', 'Mix Fold 3', 'Mix Fold 4', 'Note 12 Pro', 'Pad 6 Max 14',
    'Pad 6 Pro', 'Pad 6S Pro 12.4"', 'Pocophone F1', 'Qin 1s+', 'Qin 2', 'Qin 2 Pro', 'Qin 3 Ultra', 'Redmi 1', 'Redmi 10',
    'Redmi 10 (2022)', 'Redmi 10 5G', 'Redmi 10 Power', 'Redmi 10 Prime', 'Redmi 10 Prime (2022)', 'Redmi 10 Prime+ 5G',
    'Redmi 10A', 'Redmi 10C', 'Redmi 10X', 'Redmi 10X Pro', 'Redmi 11 Prime', 'Redmi 12', 'Redmi 12 5G', 'Redmi 12C',
    'Redmi 13', 'Redmi 13 5G', 'Redmi 13C', 'Redmi 13C 5G', 'Redmi 14C', 'Redmi 1S', 'Redmi 2', 'Redmi 2 Pro', 'Redmi 2A',
    'Redmi 3', 'Redmi 4 Prime', 'Redmi 5', 'Redmi 5 Plus', 'Redmi 6 Pro Extreme', 'Redmi 7A', 'Redmi 8', 'Redmi 8A',
    'Redmi 9', 'Redmi 9 Power', 'Redmi 9 Prime', 'Redmi 9A', 'Redmi 9AT', 'Redmi 9C', 'Redmi 9C NFC', 'Redmi 9i',
    'Redmi 9i Sport', 'Redmi 9T', 'Redmi 9T NFC', 'Redmi A1', 'Redmi A1+', 'Redmi A2', 'Redmi A2+', 'Redmi A3', 'Redmi A3x',
    'Redmi Go', 'Redmi K20', 'Redmi K20 Pro', 'Redmi K20 Pro Premium Edition', 'Redmi K30 4G', 'Redmi K30 5G',
    'Redmi K30 Pro Zoom', 'Redmi K30 Pro Zoom Edition', 'Redmi K30 Ultra', 'Redmi K30i 5G', 'Redmi K30S Ultra', 'Redmi K40',
    'Redmi K40 Gaming', 'Redmi K40 Pro', 'Redmi K40 Pro+', 'Redmi K40S', 'Redmi K50', 'Redmi K50 Pro', 'Redmi K50 Ultra',
    'Redmi K50G', 'Redmi K50i', 'Redmi K60', 'Redmi K60 Pro', 'Redmi K60 Ultra', 'Redmi K60E', 'Redmi K70', 'Redmi K70 Pro',
    'Redmi K70 Ultra', 'Redmi K70E', 'Redmi K80 Pro', 'Redmi Note', 'Redmi Note 10', 'Redmi Note 10 5G', 'Redmi Note 10 JE',
    'Redmi Note 10 Lite', 'Redmi Note 10 Pro', 'Redmi Note 10S', 'Redmi Note 10T', 'Redmi Note 10T 5G', 'Redmi Note 10X',
    'Redmi Note 11 4G', 'Redmi Note 11 5G', 'Redmi Note 11 Pro', 'Redmi Note 11 Pro 5G', 'Redmi Note 11 Pro+',
    'Redmi Note 11 Pro+ 5G', 'Redmi Note 11 SE', 'Redmi Note 11E', 'Redmi Note 11E Pro', 'Redmi Note 11R 5G',
    'Redmi Note 11S', 'Redmi Note 11S 5G', 'Redmi Note 11T 5G', 'Redmi Note 11T Pro', 'Redmi Note 11T Pro+',
    'Redmi Note 12', 'Redmi Note 12 Discovery', 'Redmi Note 12 Pro', 'Redmi Note 12 Pro DE', 'Redmi Note 12 Pro Speed',
    'Redmi Note 12 Pro+', 'Redmi Note 12 Pro+ 5G', 'Redmi Note 12R', 'Redmi Note 12R Pro', 'Redmi Note 12S',
    'Redmi Note 12T', 'Redmi Note 12T Pro', 'Redmi Note 13', 'Redmi Note 13 5G', 'Redmi Note 13 Pro',
    'Redmi Note 13 Pro 5G', 'Redmi Note 13 Pro+', 'Redmi Note 13 Pro+ 5G', 'Redmi Note 13R', 'Redmi Note 13R Pro',
    'Redmi Note 14', 'Redmi Note 14 5G', 'Redmi Note 14 Pro', 'Redmi Note 14 Pro 5G', 'Redmi Note 14 Pro+', 'Redmi Note 3',
    'Redmi Note 5A Lite', 'Redmi Note 5A Prime', 'Redmi Note 7', 'Redmi Note 7 Pro', 'Redmi Note 7S', 'Redmi Note 8',
    'Redmi Note 8 (2021)', 'Redmi Note 8 Pro', 'Redmi Note 8T', 'Redmi Note 9', 'Redmi Note 9 5G', 'Redmi Note 9 Pro',
    'Redmi Note 9 Pro 5G', 'Redmi Note 9 Pro Max', 'Redmi Note 9S', 'Redmi Note 9T 5G', 'Redmi Pad', 'Redmi Pad Pro',
    'Redmi Pad Pro 5G', 'Redmi Pad SE', 'Redmi Pad SE 8.7"', 'Redmi Turbo 3', 'Redmi Y1 Lite'
)


def get_version(options: Options) -> AndroidVersion:
    filterer = Filterer(versions)

    if options.version_ranges and 'android' in options.version_ranges:
        filterer.version_range(options.version_ranges['android'])

    if options.weighted_versions:
        filterer.weighted_versions(max_range=3)

    choice: AndroidVersion = random.choice(filterer.versions)
    choice.build_number = choice.build_number.replace('{d}', '{:02d}{:02d}{:02d}'.format(random.randint(22, 25), random.randint(0, 12), random.randint(0, 29)))
    choice.build_number = choice.build_number.replace('{v}', '{}'.format(random.randint(1, 255)))
    choice.platform_model = random.choice(platform_models)
    return choice



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""


def ch_brand_list(brand_list: list):
    serialized = []
    for _dict in brand_list:
        serialized.append('"' + _dict['brand'] + '";v="' + _dict['version'] + '"')

    return ', '.join(serialized)


def ch_bool(val: bool):
    return '?1' if val else '?0'


def ch_string(val: str):
    return '"' + val + '"'



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import random
from typing import List

from ..filterer import Filterer
from ..version import Version
from ...options import Options

# User agent cap on macOS
# https://groups.google.com/a/chromium.org/g/blink-dev/c/hAI4QoX6rEo/m/qQNPThr0AAAJ

# https://developer.apple.com/news/releases/
# https://support.apple.com/en-us/HT201222
versions: List[Version] = [
    Version(major=10, minor=11, build=(0, 6)),
    Version(major=10, minor=12, build=(0, 6)),
    Version(major=10, minor=13, build=(0, 6)),
    Version(major=10, minor=14, build=(0, 6)),
    Version(major=10, minor=15, build=(0, 7)),
    Version(major=11, minor=0, build=0),
    Version(major=11, minor=2, build=(0, 3)),
    Version(major=11, minor=3, build=(0, 1)),
    Version(major=11, minor=5, build=(0, 2)),
    Version(major=11, minor=6, build=(0, 6)),
    Version(major=12, minor=0, build=(0, 1)),
    Version(major=12, minor=2, build=(0, 1)),
    Version(major=12, minor=3, build=(0, 1)),
    Version(major=12, minor=4, build=0),
    Version(major=12, minor=5, build=(0, 1)),
    Version(major=12, minor=6, build=(0, 4)),
    Version(major=12, minor=7, build=(0, 5)),
    Version(major=13, minor=0, build=(0, 1)),
    Version(major=13, minor=1, build=0),
    Version(major=13, minor=2, build=(0, 1)),
    Version(major=13, minor=3, build=(0, 1)),
    Version(major=13, minor=4, build=(0, 1)),
    Version(major=13, minor=5, build=(0, 2)),
    Version(major=14, minor=0, build=(0, 1)),
    Version(major=14, minor=1, build=(0, 2)),
    Version(major=14, minor=2, build=(0, 1)),
    Version(major=14, minor=3, build=(0, 1)),
    Version(major=14, minor=4, build=(0, 1)),
    Version(major=14, minor=5, build=0),
    Version(major=14, minor=6, build=(0, 1)),
    Version(major=15, minor=0, build=(0, 1)),
    Version(major=15, minor=1, build=(0, 1)),
    Version(major=15, minor=2, build=(0, 1)),
    Version(major=15, minor=3, build=(0, 1)),
    Version(major=15, minor=4, build=(0, 1)),
]


def get_version(options: Options) -> Version:
    filterer = Filterer(versions)

    if options.version_ranges and 'macos' in options.version_ranges:
        filterer.version_range(options.version_ranges['macos'])

    if options.weighted_versions:
        filterer.weighted_versions()

    return random.choice(filterer.versions)



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import random
from typing import List

from ..filterer import Filterer
from ..version import Version, ChromiumVersion
from ...options import Options

# https://developer.apple.com/documentation/safari-release-notes
versions: List[ChromiumVersion] = [
    ChromiumVersion(Version(major=10, minor=0), webkit=Version(major=602, minor=4, build=8)),
    ChromiumVersion(Version(major=11, minor=0), webkit=Version(major=604, minor=1, build=38)),
    ChromiumVersion(Version(major=12, minor=(0, 1)), webkit=Version(major=605, minor=1, build=15)),
    ChromiumVersion(Version(major=13, minor=(0, 1)), webkit=Version(major=605, minor=1, build=15)),
    ChromiumVersion(Version(major=14, minor=(0, 1)), webkit=Version(major=605, minor=1, build=15)),
    ChromiumVersion(Version(major=15, minor=(0, 6)), webkit=Version(major=605, minor=1, build=15)),
    ChromiumVersion(Version(major=16, minor=(0, 6)), webkit=Version(major=605, minor=1, build=15)),
    ChromiumVersion(Version(major=17, minor=(0, 6)), webkit=Version(major=605, minor=1, build=15)),
    ChromiumVersion(Version(major=18, minor=(0, 4)), webkit=Version(major=605, minor=1, build=15)),
]


def get_version(options: Options) -> ChromiumVersion:
    filterer = Filterer(versions)

    if options.version_ranges and 'safari' in options.version_ranges:
        filterer.version_range(options.version_ranges['safari'])

    if options.weighted_versions:
        filterer.weighted_versions(max_range=3)

    return random.choice(filterer.versions)



import random

from ..data.version import VersionRange


class Filterer:
    def __init__(self, versions):
        self.versions = versions

    def version_range(self, v_range: VersionRange):
        if not isinstance(v_range, VersionRange):
            raise TypeError('Parameter v_range must be type of VersionRange')

        filtered = v_range.filter(self.versions)
        if len(filtered) > 0:
            self.versions = filtered

    def weighted_versions(self, weights=None, max_range=4):
        if weights is None:
            weights = [1.0] * len(self.versions)
            for i in range(1, min(max_range, len(self.versions))):
                weights[-i] = 10.0 - i

        self.versions = random.choices(self.versions, weights=weights, k=1)



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import random
from typing import List

from ..filterer import Filterer
from ..version import Version, WindowsVersion
from ...options import Options

# https://learn.microsoft.com/en-us/windows/win32/sysinfo/operating-system-version
# https://learn.microsoft.com/en-us/microsoft-edge/web-platform/how-to-detect-win11
versions: List[WindowsVersion] = [
    WindowsVersion(Version(major=6, minor=1), ch_platform=Version(major=0)),
    WindowsVersion(Version(major=6, minor=2), ch_platform=Version(major=0)),
    WindowsVersion(Version(major=6, minor=3), ch_platform=Version(major=0)),
    WindowsVersion(Version(major=10, minor=0), ch_platform=Version(major=10)),
    WindowsVersion(Version(major=10, minor=0), ch_platform=Version(major=(13, 15))),
]


def get_version(options: Options) -> WindowsVersion:
    filterer = Filterer(versions)

    if options.version_ranges and 'windows' in options.version_ranges:
        filterer.version_range(options.version_ranges['windows'])

    if options.weighted_versions:
        weights = [1.0] * len(versions)
        # https://gs.statcounter.com/os-version-market-share/windows/desktop/worldwide
        weights[-2] = 10.0
        weights[-1] = 7.0
        filterer.weighted_versions(weights=weights)

    return random.choice(filterer.versions)



"""
Random User-Agent
Copyright: 2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

import src.ua_generator as ua_generator
from src.ua_generator.data.version import Version, VersionRange
from src.ua_generator.options import Options


class TestVersionRange(unittest.TestCase):
    def test_version_range(self):
        version_range = VersionRange(1, 2)
        self.assertIsNotNone(version_range)
        self.assertIsNotNone(version_range.min_version)
        self.assertIsNotNone(version_range.max_version)
        self.assertIsInstance(version_range.min_version, Version)
        self.assertIsInstance(version_range.max_version, Version)
        self.assertEqual(version_range.min_version.format(), Version(1).format())
        self.assertEqual(version_range.max_version.format(), Version(2).format())
        self.assertEqual(version_range.min_version.to_tuple(), Version(1).to_tuple())
        self.assertEqual(version_range.max_version.to_tuple(), Version(2).to_tuple())
        self.assertEqual(version_range.min_version.format(partitions=4), '1.0.0.0')
        self.assertEqual(version_range.max_version.format(partitions=4), '2.0.0.0')
        self.assertTrue(version_range.max_version.to_tuple() > version_range.min_version.to_tuple())
        self.assertTrue(version_range.max_version > version_range.min_version)

    def test_version_range_2(self):
        version_range = VersionRange(Version(major=100, minor=0, build=2), 101)
        self.assertIsNotNone(version_range)
        self.assertIsNotNone(version_range.min_version)
        self.assertIsNotNone(version_range.max_version)
        self.assertIsInstance(version_range.min_version, Version)
        self.assertIsInstance(version_range.max_version, Version)
        self.assertEqual(version_range.min_version.format(), Version(major=100, minor=0, build=2).format())
        self.assertEqual(version_range.max_version.format(), Version(101).format())
        self.assertEqual(version_range.min_version.to_tuple(), Version(major=100, minor=0, build=2).to_tuple())
        self.assertEqual(version_range.max_version.to_tuple(), Version(101).to_tuple())
        self.assertEqual(version_range.min_version.format(partitions=4), '100.0.2.0')
        self.assertEqual(version_range.max_version.format(partitions=4), '101.0.0.0')
        self.assertTrue(version_range.max_version.to_tuple() > version_range.min_version.to_tuple())
        self.assertTrue(version_range.max_version > version_range.min_version)

    def test_version_range_3(self):
        # MUST be valid version range
        CHROME_MIN = 124
        CHROME_MAX = 127

        for i in range(0, 100):
            options = Options(
                version_ranges={
                    'chrome': VersionRange(min_version=CHROME_MIN, max_version=CHROME_MAX),
                })
            ua = ua_generator.generate(browser='chrome', options=options)
            self.assertTrue(ua.browser == 'chrome')
            self.assertIsNotNone(ua.generator.browser_version)
            self.assertTrue(CHROME_MIN <= ua.generator.browser_version.major <= CHROME_MAX)

    def test_version_range_4(self):
        # MUST be valid version range
        FIREFOX_MIN = 125
        FIREFOX_MAX = 129

        for i in range(0, 100):
            options = Options(version_ranges={
                'firefox': VersionRange(min_version=FIREFOX_MIN, max_version=FIREFOX_MAX),
            })
            ua = ua_generator.generate(browser='firefox', options=options)
            self.assertTrue(ua.browser == 'firefox')
            self.assertIsNotNone(ua.generator.browser_version)
            self.assertTrue(FIREFOX_MIN <= ua.generator.browser_version.major <= FIREFOX_MAX)

    def test_version_range_multiple(self):
        # MUST be valid version range
        CHROME_MIN = 124
        CHROME_MAX = 127
        FIREFOX_MIN = 125
        FIREFOX_MAX = 129

        for i in range(0, 100):
            options = Options(version_ranges={
                'chrome': VersionRange(min_version=CHROME_MIN, max_version=CHROME_MAX),
                'firefox': VersionRange(min_version=FIREFOX_MIN, max_version=FIREFOX_MAX),
            })
            ua = ua_generator.generate(browser=('chrome', 'firefox'), options=options)
            self.assertIn(ua.browser, ('chrome', 'firefox'))
            self.assertIsNotNone(ua.generator.browser_version)
            if ua.browser == 'chrome':
                self.assertTrue(CHROME_MIN <= ua.generator.browser_version.major <= CHROME_MAX)
            if ua.browser == 'firefox':
                self.assertTrue(FIREFOX_MIN <= ua.generator.browser_version.major <= FIREFOX_MAX)

    def test_version_range_5(self):
        # MUST be valid version range
        MACOS_MIN = 12
        MACOS_MAX = 14

        for i in range(0, 100):
            options = Options(version_ranges={
                'macos': VersionRange(min_version=MACOS_MIN, max_version=MACOS_MAX),
            })
            ua = ua_generator.generate(platform='macos', options=options)
            self.assertTrue(ua.platform == 'macos')
            self.assertIsNotNone(ua.generator.platform_version)
            self.assertTrue(MACOS_MIN <= ua.generator.platform_version.major <= MACOS_MAX)

    def test_version_range_min_only(self):
        # MUST be valid version range
        CHROME_MIN = 124

        for i in range(0, 100):
            options = Options(
                version_ranges={
                    'chrome': VersionRange(min_version=CHROME_MIN),
                })
            ua = ua_generator.generate(browser='chrome', options=options)
            self.assertTrue(ua.browser == 'chrome')
            self.assertIsNotNone(ua.generator.browser_version)
            self.assertTrue(CHROME_MIN <= ua.generator.browser_version.major)

    def test_version_range_max_only(self):
        # MUST be valid version range
        CHROME_MAX = 125

        for i in range(0, 100):
            options = Options(
                version_ranges={
                    'chrome': VersionRange(max_version=CHROME_MAX),
                })
            ua = ua_generator.generate(browser='chrome', options=options)
            self.assertTrue(ua.browser == 'chrome')
            self.assertIsNotNone(ua.generator.browser_version)
            self.assertTrue(ua.generator.browser_version.major <= CHROME_MAX)

    def test_version_range_invalid(self):
        # MUST be INVALID version range
        EDGE_MIN_INVALID = 1
        EDGE_MAX_INVALID = 2

        for i in range(0, 100):
            options = Options(version_ranges={
                'edge': VersionRange(min_version=EDGE_MIN_INVALID, max_version=EDGE_MAX_INVALID),
            })
            ua = ua_generator.generate(browser='edge', options=options)
            self.assertTrue(ua.browser == 'edge')
            self.assertIsNotNone(ua.generator.browser_version)
            self.assertFalse(EDGE_MIN_INVALID <= ua.generator.browser_version.major <= EDGE_MAX_INVALID)


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

import src.ua_generator as ua_generator
from src.ua_generator.data import BROWSERS_SUPPORT_CH


class TestHeaders(unittest.TestCase):
    def test_default(self):
        for i in range(0, 100):
            ua = ua_generator.generate()
            self.assertIsNotNone(ua.headers)
            self.assertTrue('user-agent' in ua.headers.get())

    def test_client_hints(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=BROWSERS_SUPPORT_CH)
            self.assertIsNotNone(ua.headers)
            self.assertTrue('sec-ch-ua' in ua.headers.get())
            self.assertTrue('sec-ch-ua-mobile' in ua.headers.get())
            self.assertTrue('sec-ch-ua-platform' in ua.headers.get())

    def test_client_hints_not_exists(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser='firefox')
            self.assertIsNotNone(ua.headers)
            self.assertFalse('sec-ch-ua' in ua.headers.get())
            self.assertFalse('sec-ch-ua-mobile' in ua.headers.get())
            self.assertFalse('sec-ch-ua-platform' in ua.headers.get())

    def test_accept_ch(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=BROWSERS_SUPPORT_CH)

            ua.headers.accept_ch('Sec-CH-UA-Platform-Version, Sec-CH-UA-Full-Version-List')
            self.assertTrue('sec-ch-ua-platform-version' in ua.headers.get())
            self.assertTrue('sec-ch-ua-full-version-list' in ua.headers.get())

            ua.headers.accept_ch('Sec-CH-UA-Bitness, Sec-CH-UA-Arch')
            self.assertTrue('sec-ch-ua-bitness' in ua.headers.get())
            self.assertTrue('sec-ch-ua-arch' in ua.headers.get())
            self.assertFalse('sec-ch-ua-platform-version' in ua.headers.get())
            self.assertFalse('sec-ch-ua-full-version-list' in ua.headers.get())

            ua.headers.accept_ch('Sec-CH-UA-Model, Sec-CH-UA-WoW64')
            self.assertTrue('sec-ch-ua-model' in ua.headers.get())
            self.assertTrue('sec-ch-ua-wow64' in ua.headers.get())

    def test_reset(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=BROWSERS_SUPPORT_CH)

            ua.headers.add('sec-ch-ua-bitness')
            self.assertTrue('user-agent' in ua.headers.get())
            self.assertTrue('sec-ch-ua' in ua.headers.get())
            self.assertTrue('sec-ch-ua-bitness' in ua.headers.get())

            ua.headers.reset()
            self.assertTrue('user-agent' in ua.headers.get())
            self.assertTrue('sec-ch-ua' in ua.headers.get())
            self.assertFalse('sec-ch-ua-bitness' in ua.headers.get())

    def test_accept_ch_not_exists(self):
        for i in range(0, 100):
            ua = ua_generator.generate(browser=BROWSERS_SUPPORT_CH)
            ua.headers.accept_ch('Sec-CH-Example')
            self.assertFalse('sec-ch-ua-platform-version' in ua.headers.get())
            self.assertFalse('sec-ch-ua-full-version-list' in ua.headers.get())


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""
from random import Random

from . import serialization
from .data import generator, PLATFORMS_MOBILE
from .data.version import AndroidVersion, WindowsVersion


# https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Sec-CH-UA
# https://wicg.github.io/ua-client-hints/#http-ua-hints
class ClientHints:
    # Type hinting only
    mobile: str
    platform: str
    platform_version: str
    brands: str
    brands_full_version_list: str
    bitness: str
    architecture: str
    model: str
    wow64: str

    def __init__(self, gen: generator.Generator):
        self.__generator = gen
        self.__cache = {}

    def get_mobile(self) -> bool:
        return self.__generator.platform in PLATFORMS_MOBILE

    def get_platform(self) -> str:
        platform = self.__generator.platform

        if platform == 'ios':
            platform = 'iOS'
        elif platform == 'macos':
            platform = 'macOS'
        else:
            platform = platform.title()

        return platform

    def get_platform_version(self) -> str:
        if type(self.__generator.platform_version) is WindowsVersion:
            return self.__generator.platform_version.ch_platform.format(partitions=3)

        return str(self.__generator.platform_version)

    def get_brands(self, full_version_list: bool = False) -> list:
        brand_list = [{'brand': 'Not A(Brand', 'version': '99.0.0.0' if full_version_list else '99'}]

        if self.__generator.browser == 'chrome':
            browser_version = self.get_browser_version(full_version=full_version_list)
            brand_list.append({'brand': 'Chromium', 'version': browser_version})
            brand_list.append({'brand': 'Google Chrome', 'version': browser_version})
        elif self.__generator.browser == 'edge':
            browser_version = self.get_browser_version(full_version=full_version_list)
            brand_list.append({'brand': 'Chromium', 'version': browser_version})
            brand_list.append({'brand': 'Microsoft Edge', 'version': browser_version})

        return brand_list

    def get_browser_version(self, full_version: bool = True) -> str:
        if full_version:
            return str(self.__generator.browser_version)
        else:
            return str(self.__generator.browser_version.major)

    def get_bitness(self) -> str:
        if self.__generator.platform == 'android':
            _random = Random(self.__generator.user_agent)
            return _random.choice(('32', '64', '32', '32'))

        return '64'

    def get_architecture(self) -> str:
        if self.__generator.platform == 'android' or self.__generator.platform == 'ios':
            return 'arm'
        elif self.__generator.platform == 'macos':
            _random = Random(self.__generator.user_agent)
            return _random.choice(('arm', 'x86', 'arm', 'arm'))

        return 'x86'

    def get_model(self) -> str:
        if type(self.__generator.platform_version) is AndroidVersion:
            return self.__generator.platform_version.platform_model or ''

        return ''

    def get_wow64(self) -> bool:
        return self.__generator.platform == 'windows'

    def __getattr__(self, name) -> str:
        if name in self.__cache:
            return self.__cache[name]

        if name == 'mobile':
            self.__cache[name] = serialization.ch_bool(self.get_mobile())
        elif name == 'platform':
            self.__cache[name] = serialization.ch_string(self.get_platform())
        elif name == 'platform_version':
            self.__cache[name] = serialization.ch_string(self.get_platform_version())
        elif name == 'brands':
            self.__cache[name] = serialization.ch_brand_list(self.get_brands())
        elif name == 'brands_full_version_list':
            self.__cache[name] = serialization.ch_brand_list(self.get_brands(full_version_list=True))
        elif name == 'bitness':
            self.__cache[name] = serialization.ch_string(self.get_bitness())
        elif name == 'architecture':
            self.__cache[name] = serialization.ch_string(self.get_architecture())
        elif name == 'model':
            self.__cache[name] = serialization.ch_string(self.get_model())
        elif name == 'wow64':
            self.__cache[name] = serialization.ch_bool(self.get_wow64())
        else:
            raise AttributeError('Invalid attribute: {}'.format(name))

        return self.__cache[name]

    def __str__(self):
        return self.brands

    def __repr__(self):
        return (f"ClientHints("
                f"mobile={self.mobile}, "
                f"platform={self.platform}, "
                f"platform_version={self.platform_version}, "
                f"brands_full_version_list={self.brands_full_version_list}, "
                f"bitness={self.bitness}, "
                f"architecture={self.architecture}, "
                f"model={self.model}, "
                f"wow64={self.wow64}"
                f")")



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import random
from typing import List

from ..filterer import Filterer
from ..version import Version
from ...options import Options

# https://git.kernel.org/pub/scm/linux/kernel/git/stable/linux.git/refs/
versions: List[Version] = [
    Version(major=5, minor=0, build=(0, 21)),
    Version(major=5, minor=1, build=(0, 21)),
    Version(major=5, minor=2, build=(0, 20)),
    Version(major=5, minor=3, build=(0, 18)),
    Version(major=5, minor=4, build=(0, 184)),
    Version(major=5, minor=5, build=(0, 19)),
    Version(major=5, minor=6, build=(0, 19)),
    Version(major=5, minor=7, build=(0, 19)),
    Version(major=5, minor=8, build=(0, 18)),
    Version(major=5, minor=9, build=(0, 16)),
    Version(major=5, minor=10, build=(0, 105)),
    Version(major=5, minor=11, build=(0, 22)),
    Version(major=5, minor=12, build=(0, 19)),
    Version(major=5, minor=13, build=(0, 19)),
    Version(major=5, minor=14, build=(0, 21)),
    Version(major=5, minor=15, build=(0, 103)),
    Version(major=5, minor=16, build=(0, 20)),
    Version(major=5, minor=17, build=(0, 15)),
    Version(major=5, minor=18, build=(0, 19)),
    Version(major=5, minor=19, build=(0, 17)),
    Version(major=6, minor=0, build=(0, 19)),
    Version(major=6, minor=1, build=(0, 78)),
    Version(major=6, minor=2, build=(0, 16)),
    Version(major=6, minor=3, build=(0, 13)),
    Version(major=6, minor=4, build=(0, 16)),
    Version(major=6, minor=5, build=(0, 13)),
    Version(major=6, minor=6, build=(0, 17)),
    Version(major=6, minor=7, build=(0, 5)),
]


def get_version(options: Options) -> Version:
    filterer = Filterer(versions)

    if options.version_ranges and 'linux' in options.version_ranges:
        filterer.version_range(options.version_ranges['linux'])

    if options.weighted_versions:
        filterer.weighted_versions()

    return random.choice(filterer.versions)



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0
"""
from .client_hints import ClientHints
from .data import BROWSERS_SUPPORT_CH
from .data.generator import Generator


# https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept-CH
class Headers:

    def __init__(self, gen: Generator, ch: ClientHints):
        self.__generator = gen
        self.__client_hints = ch
        self.__is_generated = False
        self.__headers: dict[str, str] = {}

    def reset(self):
        self.__is_generated = True
        self.__headers = {
            'user-agent': self.__generator.user_agent,
        }

        # https://developer.mozilla.org/en-US/docs/Web/HTTP/Client_hints#low_entropy_hints
        if self.__generator.browser in BROWSERS_SUPPORT_CH:
            self.add('sec-ch-ua')
            self.add('sec-ch-ua-mobile')
            self.add('sec-ch-ua-platform')

    def add(self, key: str):
        if not self.__is_generated:
            self.reset()

        if key == 'sec-ch-ua':
            self.__headers[key] = self.__client_hints.brands
        elif key == 'sec-ch-ua-full-version-list':
            self.__headers[key] = self.__client_hints.brands_full_version_list
        elif key == 'sec-ch-ua-platform':
            self.__headers[key] = self.__client_hints.platform
        elif key == 'sec-ch-ua-platform-version':
            self.__headers[key] = self.__client_hints.platform_version
        elif key == 'sec-ch-ua-mobile':
            self.__headers[key] = self.__client_hints.mobile
        elif key == 'sec-ch-ua-bitness':
            self.__headers[key] = self.__client_hints.bitness
        elif key == 'sec-ch-ua-arch':
            self.__headers[key] = self.__client_hints.architecture
        elif key == 'sec-ch-ua-model':
            self.__headers[key] = self.__client_hints.model
        elif key == 'sec-ch-ua-wow64':
            self.__headers[key] = self.__client_hints.wow64

    def accept_ch(self, val: str):
        self.reset()

        if self.__generator.browser not in BROWSERS_SUPPORT_CH:
            return

        requested_hints = val.split(',')
        for hint in requested_hints:
            self.add(hint.strip().lower())

    def get(self) -> dict[str, str]:
        if not self.__is_generated:
            self.reset()

        return self.__headers

    def __str__(self):
        text = ""
        for k, v in self.get().items():
            text += f"{k}: {v}\n"
        return text



"""
Random User-Agent
Copyright: 2024-2025 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import unittest

from src.ua_generator.data.version import Version, WindowsVersion, AndroidVersion, ChromiumVersion


class TestVersion(unittest.TestCase):
    def test_version(self):
        version = Version(major=1, minor=2, build=3, patch=4)
        self.assertEqual(version.format(), '1.2.3.4')
        self.assertEqual(version.format(partitions=4), '1.2.3.4')
        self.assertEqual(version.format(partitions=3), '1.2.3')
        self.assertEqual(version.format(partitions=2), '1.2')
        self.assertEqual(version.format(partitions=1), '1')
        self.assertEqual(version.to_tuple(), (1, 2, 3, 4))

    def test_version_2(self):
        version = Version(major=1, minor=2, build=3, patch=None)
        self.assertEqual(version.format(), '1.2.3')
        self.assertEqual(version.format(partitions=4), '1.2.3.0')
        self.assertEqual(version.format(partitions=3), '1.2.3')
        self.assertEqual(version.format(partitions=2), '1.2')
        self.assertEqual(version.format(partitions=1), '1')
        self.assertEqual(version.to_tuple(), (1, 2, 3, 0))

    def test_version_3(self):
        version = Version(major=1, minor=2, build=None, patch=0)
        self.assertEqual(version.format(), '1.2.0.0')
        self.assertEqual(version.format(partitions=4), '1.2.0.0')
        self.assertEqual(version.format(partitions=3), '1.2.0')
        self.assertEqual(version.format(partitions=2), '1.2')
        self.assertEqual(version.format(partitions=1), '1')
        self.assertEqual(version.to_tuple(), (1, 2, 0, 0))

    def test_version_4(self):
        version = Version(major=1, minor=2, patch=4)
        self.assertEqual(version.format(), '1.2.0.4')
        self.assertEqual(version.format(partitions=4), '1.2.0.4')
        self.assertEqual(version.format(partitions=3), '1.2.0')
        self.assertEqual(version.format(partitions=2), '1.2')
        self.assertEqual(version.format(partitions=1), '1')
        self.assertEqual(version.to_tuple(), (1, 2, 0, 4))

    def test_version_5(self):
        version = Version(major=0, build=3)
        self.assertEqual(version.format(), '0.0.3')
        self.assertEqual(version.format(partitions=4), '0.0.3.0')
        self.assertEqual(version.format(partitions=3), '0.0.3')
        self.assertEqual(version.format(partitions=2), '0.0')
        self.assertEqual(version.format(partitions=1), '0')
        self.assertEqual(version.to_tuple(), (0, 0, 3, 0))

    def test_version_6(self):
        version = Version(major=1, minor=2)
        self.assertEqual(version.format(), '1.2')
        self.assertEqual(version.format(partitions=3), '1.2.0')
        self.assertEqual(version.to_tuple(), (1, 2, 0, 0))

    def test_version_7(self):
        version = Version(major=1)
        self.assertEqual(version.format(), '1')
        self.assertEqual(version.format(partitions=2), '1.0')
        self.assertEqual(version.to_tuple(), (1, 0, 0, 0))

    def test_version_none(self):
        version = Version(major=None, minor=None, build=None, patch=None)
        self.assertEqual(version.format(), '')
        self.assertEqual(version.format(partitions=1), '0')
        self.assertEqual(version.format(partitions=2), '0.0')
        self.assertEqual(version.to_tuple(), (0, 0, 0, 0))

    def test_version_separator(self):
        version = Version(major=1, minor=2, build=3, patch=4)
        self.assertEqual(version.format(separator='_'), '1_2_3_4')
        self.assertEqual(version.format(partitions=4, separator='_'), '1_2_3_4')
        self.assertEqual(version.format(partitions=3, separator='_'), '1_2_3')
        self.assertEqual(version.format(partitions=2, separator='_'), '1_2')
        self.assertEqual(version.format(partitions=1, separator='_'), '1')

    def test_version_separator_2(self):
        version = Version(major=1, minor=2, build=3, patch=None)
        self.assertEqual(version.format(separator='_'), '1_2_3')
        self.assertEqual(version.format(partitions=4, separator='_'), '1_2_3_0')
        self.assertEqual(version.format(partitions=3, separator='_'), '1_2_3')
        self.assertEqual(version.format(partitions=2, separator='_'), '1_2')
        self.assertEqual(version.format(partitions=1, separator='_'), '1')

    def test_version_separator_3(self):
        version = Version(major=1, minor=2, build=3, patch=0)
        self.assertEqual(version.format(separator='_'), '1_2_3_0')
        self.assertEqual(version.format(partitions=4, separator='_'), '1_2_3_0')
        self.assertEqual(version.format(partitions=3, separator='_'), '1_2_3')
        self.assertEqual(version.format(partitions=2, separator='_'), '1_2')
        self.assertEqual(version.format(partitions=1, separator='_'), '1')

    def test_version_trim_zero(self):
        version = Version(major=1, minor=2, build=0, patch=0)
        self.assertEqual(version.format(trim_zero=True), '1.2')

    def test_version_trim_zero_2(self):
        version = Version(major=1, minor=2, build=0, patch=4)
        self.assertEqual(version.format(trim_zero=True), '1.2.0.4')

    def test_version_trim_zero_3(self):
        version = Version(major=1, minor=None, build=3, patch=4)
        self.assertEqual(version.format(partitions=2, trim_zero=True), '1')

    def test_version_range(self):
        version = Version(build=(90, 100))
        self.assertTrue(version.build >= 90 and version.build <= 100)

    def test_version_windows(self):
        version = WindowsVersion(version=Version(major=10, minor=0), ch_platform=Version(major=1, minor=2))
        self.assertEqual(version.format(partitions=4), '10.0.0.0')
        self.assertEqual(version.ch_platform.format(partitions=4), '1.2.0.0')

    def test_version_windows_range(self):
        version = WindowsVersion(version=Version(major=10, minor=0), ch_platform=Version(major=(1, 10)))
        self.assertEqual(version.format(partitions=4), '10.0.0.0')
        self.assertTrue(version.ch_platform.major >= 1 and version.ch_platform.major <= 10)

    def test_version_android(self):
        version = AndroidVersion(version=Version(), build_numbers=('foo', 'bar'))
        self.assertIn(version.build_number, ('foo', 'bar'))

    def test_version_android_2(self):
        version = AndroidVersion(version=Version(), build_numbers=['foo', 'bar'])
        self.assertIn(version.build_number, ['foo', 'bar'])

    def test_version_android_3(self):
        version = AndroidVersion(version=Version(), build_numbers='foo')
        self.assertEqual(version.build_number, 'foo')

    def test_version_chromium(self):
        version = ChromiumVersion(Version(major=1, minor=2, build=3, patch=4), webkit=Version(537, 36))
        self.assertEqual(version.format(partitions=4), '1.2.3.4')
        self.assertEqual(version.webkit.format(), '537.36')

    def test_version_comparison(self):
        version_1 = Version(major=1, minor=2)
        version_2 = Version(major=1, minor=3)
        self.assertFalse(version_1 == version_2)
        self.assertTrue(version_1 != version_2)
        self.assertTrue(version_1 < version_2)
        self.assertFalse(version_1 > version_2)
        self.assertTrue(version_1 <= version_2)
        self.assertFalse(version_1 >= version_2)

    def test_version_comparison_2(self):
        version_1 = Version(major=1, minor=0)
        version_2 = Version(major=1, minor=0)
        self.assertTrue(version_1 == version_2)
        self.assertFalse(version_1 != version_2)
        self.assertFalse(version_1 < version_2)
        self.assertFalse(version_1 > version_2)
        self.assertTrue(version_1 <= version_2)
        self.assertTrue(version_1 >= version_2)


if __name__ == '__main__':
    unittest.main()



"""
Random User-Agent
Copyright: 2022-2024 Ekin Karadeniz (github.com/iamdual)
License: Apache License 2.0 
"""
import random
from typing import List

from ..filterer import Filterer
from ..version import Version
from ...options import Options

# https://developer.apple.com/documentation/ios-ipados-release-notes
# https://developer.apple.com/news/releases/
# https://support.apple.com/en-us/HT201222
versions: List[Version] = [
    Version(major=14, minor=0, build=(0, 1)),
    Version(major=14, minor=1, build=0),
    Version(major=14, minor=2, build=(0, 1)),
    Version(major=14, minor=3, build=0),
    Version(major=14, minor=4, build=(0, 2)),
    Version(major=14, minor=5, build=(0, 1)),
    Version(major=14, minor=6, build=0),
    Version(major=14, minor=7, build=(0, 1)),
    Version(major=15, minor=0, build=(0, 2)),
    Version(major=15, minor=1, build=(0, 1)),
    Version(major=15, minor=2, build=(0, 1)),
    Version(major=15, minor=3, build=(0, 1)),
    Version(major=15, minor=4, build=(0, 1)),
    Version(major=15, minor=5, build=0),
    Version(major=15, minor=6, build=(0, 1)),
    Version(major=16, minor=4, build=(0, 1)),
    Version(major=16, minor=5, build=(0, 2)),
    Version(major=16, minor=6, build=(0, 1)),
    Version(major=17, minor=0, build=(0, 3)),
    Version(major=17, minor=1, build=(0, 2)),
    Version(major=17, minor=2, build=(0, 1)),
    Version(major=17, minor=3, build=(0, 1)),
    Version(major=17, minor=4, build=(0, 1)),
    Version(major=17, minor=5, build=(0, 1)),
    Version(major=17, minor=6, build=(0, 1)),
    Version(major=18, minor=0, build=(0, 1)),
    Version(major=18, minor=1, build=(0, 1)),
    Version(major=18, minor=2, build=(0, 1)),
    Version(major=18, minor=3, build=(0, 1)),
    Version(major=18, minor=4, build=(0, 1)),
]


def get_version(options: Options) -> Version:
    filterer = Filterer(versions)

    if options.version_ranges and 'ios' in options.version_ranges:
        filterer.version_range(options.version_ranges['ios'])

    if options.weighted_versions:
        filterer.weighted_versions()

    return random.choice(filterer.versions)
